@@ -3,8 +3,10 @@ import numpy as np
 import tensorflow as tf
 import pandas as pd
 import os
+import typing as t
 from typing import Union, List, Callable
-from collections.abc import MutableMapping
+from collections.abc import MutableSequence
+
 from kgcnn.data.utils import save_pickle_file, load_pickle_file, ragged_tensor_from_nested_numpy
 from kgcnn.graph.base import GraphDict
 
@@ -13,7 +15,7 @@ module_logger = logging.getLogger(__name__)
 module_logger.setLevel(logging.INFO)
 
 
-class MemoryGraphList(MutableMapping):
+class MemoryGraphList(MutableSequence):
     r"""Class to store a list of graph dictionaries in memory.
 
     Contains a python list as property :obj:`_list`. The graph properties are defined by tensor-like (numpy) arrays
@@ -132,6 +134,23 @@ class MemoryGraphList(MutableMapping):
     def __iter__(self):
         return iter(self._list)
 
+    def __repr__(self):
+        return "<{} [{}]>".format(type(self).__name__, "" if len(self) == 0 else self[0].__repr__() + " ...")
+
+    def append(self, graph):
+        assert isinstance(graph, GraphDict), "Must append `GraphDict` to self."
+        self._list.append(graph)
+
+    def insert(self, index: int, value) -> None:
+        assert isinstance(value, GraphDict), "Must insert `GraphDict` to self."
+        self._list.insert(index, value)
+
+    def __add__(self, other):
+        assert isinstance(other, MemoryGraphList), "Must add `MemoryGraphList` to self."
+        new_list = MemoryGraphList()
+        new_list._set_internal_list(self._list + other._list)
+        return new_list
+
     def _set_internal_list(self, value: list):
         if not isinstance(value, list):
             raise TypeError("Must set list for `MemoryGraphList` internal assignment.")
@@ -162,6 +181,12 @@ class MemoryGraphList(MutableMapping):
         self._list = [GraphDict() for _ in range(length)]
         return self
 
+    def update(self, other) -> None:
+        assert isinstance(other, MemoryGraphList), "Must update `MemoryGraphList`."
+        assert len(other) == len(self), "Length of list for update must match."
+        for i in range(len(self)):
+            self[i].update(other[i])
+
     @property
     def length(self):
         """Length of list."""
@@ -172,6 +197,7 @@ class MemoryGraphList(MutableMapping):
         raise ValueError("Can not set length. Please use 'empty()' to initialize an empty list.")
 
     def _to_tensor(self, item, make_copy=True):
+        # TODO: Document this
         if not make_copy:
             self.logger.warning("At the moment always a copy is made for tensor().")
         props = self.obtain_property(item["name"])  # Will be list.
@@ -190,7 +216,7 @@ class MemoryGraphList(MutableMapping):
             items (list): List of dictionaries that specify graph properties in list via 'name' key.
                 The dict-items match the tensor input for :obj:`tf.keras.layers.Input` layers.
                 Required dict-keys should be 'name' and 'ragged'.
-                Optionally shape information can be included via 'shape'.
+                Optionally shape information can be included via 'shape' and 'dtype'.
                 E.g.: `[{'name': 'edge_indices', 'ragged': True}, {...}, ...]`.
             make_copy (bool): Whether to copy the data. Default is True.
 
@@ -263,24 +289,24 @@ class MemoryGraphList(MutableMapping):
                 item_name = item
             props = self.obtain_property(item_name)
             if props is None:
-                self.logger.warning("Can not clean property %s as it was not assigned to any graph." % item)
+                self.logger.warning("Can not clean property '%s' as it was not assigned to any graph." % item)
                 continue
             for i, x in enumerate(props):
                 # If property is neither list nor np.array
                 if x is None or not hasattr(x, "__getitem__"):
-                    self.logger.info("Property %s is not defined for graph %s." % (item_name, i))
+                    self.logger.info("Property '%s' is not defined for graph '%s'." % (item_name, i))
                     invalid_graphs.append(i)
                 elif not isinstance(x, np.ndarray):
-                    self.logger.info("Property %s is not a numpy array for graph %s." % (item_name, i))
+                    self.logger.info("Property '%s' is not a numpy array for graph '%s'." % (item_name, i))
                     invalid_graphs.append(i)
                 elif len(x.shape) > 0:
                     if len(x) <= 0:
-                        self.logger.info("Property %s is an empty list for graph %s." % (item_name, i))
+                        self.logger.info("Property '%s' is an empty list for graph '%s'." % (item_name, i))
                         invalid_graphs.append(i)
         invalid_graphs = np.unique(np.array(invalid_graphs, dtype="int"))
         invalid_graphs = np.flip(invalid_graphs)  # Need descending order for pop()
         if len(invalid_graphs) > 0:
-            self.logger.warning("Found invalid graphs for properties. Removing graphs %s." % invalid_graphs)
+            self.logger.warning("Found invalid graphs for properties. Removing graphs '%s'." % invalid_graphs)
         else:
             self.logger.info("No invalid graphs for assigned properties found.")
         # Remove from the end via pop().
@@ -592,8 +618,12 @@ class MemoryGraphDataset(MemoryGraphList):
                 else:
                     self.error("Class does not have method '%s'." % method)
 
-    def get_split_indices(self, name: str = "kfold", return_as_train_test: bool = True,
-                          shuffle: bool = True, seed: int = None):
+    def get_split_indices(self,
+                          name: str = "kfold",
+                          return_as_train_test: bool = True,
+                          shuffle: bool = True,
+                          seed: int = None
+                          ):
         """Gather split ids from split graph property and return k-fold splits.
 
         Args:
@@ -611,7 +641,6 @@ class MemoryGraphDataset(MemoryGraphList):
             if to_split - len(split_indices) + 1 > 0:
                 for _ in range(to_split - len(split_indices) + 1):
                     split_indices.append([])
-
         graphs = self.obtain_property(name)
         for i, s in enumerate(graphs):
             if s is None:
@@ -640,42 +669,103 @@ class MemoryGraphDataset(MemoryGraphList):
 
         return train_test
 
-    def get_train_test_indices(self, train: str = "train", test: str = "test", valid: str = None,
-                               split_index: Union[int, list] = 1):
-        """Get train and test indices from graph list. The 'train' and 'test' properties must be set on the graph.
-        They can also be a list of split assignment if more than one train-test split is required.
+    def get_train_test_indices(self,
+                               train: str = "train",
+                               test: str = "test",
+                               valid: t.Optional[str] = None,
+                               split_index: Union[int, list] = 1,
+                               shuffle: bool = False,
+                               seed: int = None
+                               ) -> t.List[t.Union[t.Tuple[int, int], t.Tuple[int, int, int]]]:
+        """
+        Get train and test indices from graph list.
+        The 'train' and 'test' properties must be set on the graph, and optionally an additional property
+        for the validation split may be present. All of these properties may have either of the following
+        values:
+        - The property is a boolean integer value indicating whether the corresponding element of the
+          dataset belongs to that part of the split (train / test)
+        - The property is a list containing integer split indices, where each split index present within
+          that list implies that the corresponding dataset element is part of that particular split.
+          In this case the ``split_index`` parameter may also be a list of split indices that specifies
+          for which of these split indices the train test index split is to be returned by this method.
+
+        The return value of this method is a list with the same length as the ``split_index`` parameter,
+        which by default will be 1.
 
         Args:
             train (str): Name of graph property that has train split assignment. Defaults to 'train'.
             test (str): Name of graph property that has test split assignment. Defaults to 'test'.
             valid (str): Name of graph property that has validation assignment. Defaults to None.
             split_index (int, list): Split index to get indices for. Can also be list.
+            shuffle (bool): Whether to shuffle splits. Default is True.
+            seed (int): Random seed for shuffle. Default is None.
 
         Returns:
-            list: list of train, test, validation split indices.
+            list: List of tuples (or triples) of train, test, (validation) split indices.
         """
         out_indices = []
         if not isinstance(split_index, (list, tuple)):
-            split_index = [split_index]
-        for s in [train, test, valid]:
-            if s is None:
-                out_indices.append(None)
-                continue
-            s_list = []
-            split_prop = self.obtain_property(s)
-            for j in split_index:
-                split_list = []
-                for i, x in enumerate(split_prop):
-                    if x is not None:
-                        if j in x:
-                            split_list.append(i)
-                s_list.append(split_list)
-            if len(s_list) == 1:
-                out_indices.append(s_list[0])
-            else:
-                out_indices.append(s_list[0])
+            split_index_list: t.List[int] = [split_index]
+        else:
+            split_index_list: t.List[int] = split_index
+
+        for split_index in split_index_list:
+
+            # This list will itself contain numpy arrays which are filled with graph indices of the dataset
+            # each element of this list will correspond to one property name (train, test...)
+            graph_index_split_list: t.List[np.ndarray] = []
+
+            for property_name in [train, test, valid]:
+
+                # It may be that we only seek train and test indices and not validation indices, in that
+                # case the property name for validation is None, in which case we want to skip
+                if property_name is None:
+                    continue
+
+                # This list will contain all the indices of the dataset elements (graphs) which are
+                # associated with the current iteration's split index for the current iteration's
+                # property name (train, test...)
+                graph_index_list: t.List[int] = []
+
+                # "obtain_property" returns a list which contains only the property values corresponding to
+                # the given property name for each graph inside the dataset in the same order.
+                # In this case, this is supposed to be a split list, which is a list that contains integer
+                # indices, each representing one particular dataset split. The split list of each graph
+                # only contains those split indices to which that graph is associated.
+                split_prop: t.List[t.List[int]] = self.obtain_property(property_name)
+                for index, split_list in enumerate(split_prop):
+                    if split_list is not None:
+                        if split_index in split_list:
+                            graph_index_list.append(index)
+
+                graph_index_array = np.array(graph_index_list)
+                graph_index_split_list.append(graph_index_array)
+
+            if shuffle:
+                np.random.seed(seed)
+                for graph_index_array in graph_index_split_list:
+                    np.random.shuffle(graph_index_array)
+
+            out_indices.append(graph_index_split_list)
 
         return out_indices
 
+    def relocate(self, data_directory: str = None, file_name: str = None, file_directory: str = None):
+        """Change file information. Does not copy files on disk!
+
+        Args:
+            data_directory (str): Full path to directory of the dataset. Default is None.
+            file_name (str): Generic filename for dataset to read into memory like a 'csv' file. Default is None.
+            file_directory (str): Name or relative path from :obj:`data_directory` to a directory containing sorted
+                files. Default is None.
+
+        Returns:
+            self
+        """
+        self.data_directory = data_directory
+        self.file_name = file_name
+        self.file_directory = file_directory
+        return self
+
 
 MemoryGeometricGraphDataset = MemoryGraphDataset
