@@ -188,9 +188,11 @@ def range_neighbour_lattice(coordinates: np.ndarray, lattice: np.ndarray,
     :obj:`max_neighbours`, you can set :obj:`limit_only_max_neighbours` to `True`.
 
     .. warning::
+
         All atoms should be projected back into the primitive unit cell before calculating the range connections.
 
     .. note::
+
         For periodic structure, setting :obj:`max_distance` and :obj:`max_neighbours` to `inf` would also lead
         to an infinite number of neighbours and connections. If :obj:`exclusive` is set to `False`, having either
         :obj:`max_distance` or :obj:`max_neighbours` set to `inf`, will result in an infinite number of neighbours.
@@ -238,7 +240,7 @@ def range_neighbour_lattice(coordinates: np.ndarray, lattice: np.ndarray,
     center_indices = np.indices((len(node_index), len(node_index)))
     center_indices = center_indices.transpose(np.append(np.arange(1, 3), 0))  # NxNx2
     center_dist = np.expand_dims(coordinates, axis=0) - np.expand_dims(coordinates, axis=1)  # NxNx3
-    center_image = np.zeros(center_dist.shape)
+    center_image = np.zeros(center_dist.shape, dtype="int")
     if not self_loops:
         def remove_self_loops(x):
             m = np.logical_not(np.eye(len(x), dtype="bool"))
@@ -364,3 +366,57 @@ def range_neighbour_lattice(coordinates: np.ndarray, lattice: np.ndarray,
     out_indices = dist_indices_sort[mask]
 
     return [out_indices, out_images, out_dist]
+
+
+def get_principal_moments_of_inertia(masses: np.ndarray, coordinates: np.ndarray, shift_center_of_mass: bool = True):
+    """Compute principle moments of inertia for a point-cloud with given mass.
+
+    Args:
+        masses (np.ndarray): Array of mass values.
+        coordinates (np.ndarray): Coordinates of point-cloud.
+        shift_center_of_mass (bool): Whether to shift to center of mass for inertia matrix.
+
+    Returns:
+        np.ndarray: Eigenvalues of inertia matrix. Shape is (3, ).
+    """
+
+    def translate_to_center_of_mass(m, xyz):
+        if len(m.shape) <= 1:
+            m = np.expand_dims(m, axis=-1)
+        com = (np.sum(m * xyz, axis=0, keepdims=True) / np.sum(m))
+        return xyz - com
+
+    def get_inertia_matrix(m, xyz):
+        x, y, z = xyz.T
+        Ixx = np.sum(m * (y ** 2 + z ** 2))
+        Iyy = np.sum(m * (x ** 2 + z ** 2))
+        Izz = np.sum(m * (x ** 2 + y ** 2))
+        Ixy = -np.sum(m * x * y)
+        Iyz = -np.sum(m * y * z)
+        Ixz = -np.sum(m * x * z)
+        return np.array([[Ixx, Ixy, Ixz], [Ixy, Iyy, Iyz], [Ixz, Iyz, Izz]])
+
+    coordinates = translate_to_center_of_mass(masses, coordinates) if shift_center_of_mass else coordinates
+    inertia_matrix = get_inertia_matrix(masses, coordinates)
+    pc = np.linalg.eigvals(inertia_matrix)
+    pc = np.sort(pc)
+
+    return pc
+
+
+def shift_coordinates_to_unit_cell(coordinates: np.ndarray, lattice: np.ndarray):
+    """Shift a set of coordinates into the unit cell of a periodic system.
+
+    Args:
+        coordinates (np.ndarray): Coordinate of nodes in the central primitive unit cell.
+        lattice (np.ndarray): Lattice matrix of real space lattice vectors of shape `(3, 3)`.
+            The lattice vectors must be given in rows of the matrix!
+
+    Returns:
+        np.ndarray: Coordinates shifted into unit cell defined by lattice.
+    """
+    lattice_inv = np.linalg.inv(lattice)
+    frac_coordinates = np.dot(coordinates, lattice_inv)
+    shifted_frac_coordinates = frac_coordinates % 1.0 % 1.0
+    shifted_coordinates = np.dot(shifted_frac_coordinates, lattice)
+    return shifted_coordinates
