@@ -6,7 +6,7 @@ from kgcnn.graph.adj import get_angle_indices, coordinates_to_distancematrix, in
     define_adjacency_from_distance, sort_edge_indices, get_angle, add_edges_reverse_indices, \
     rescale_edge_weights_degree_sym, add_self_loops_to_edge_indices, compute_reverse_edges_index_map, \
     distance_to_gauss_basis
-from kgcnn.graph.geom import range_neighbour_lattice
+from kgcnn.graph.geom import range_neighbour_lattice, get_principal_moments_of_inertia, shift_coordinates_to_unit_cell
 
 logging.basicConfig()  # Module logger
 module_logger = logging.getLogger(__name__)
@@ -22,7 +22,7 @@ class MakeUndirectedEdges(GraphPreProcessorBase):
     Args:
         edge_indices (str): Name of indices in dictionary. Default is "edge_indices".
         edge_attributes (str): Name of related edge values or attributes.
-            This can be a match-string or list of names. Default is "^edge_.*".
+            This can be a match-string or list of names. Default is "^edge_(?!indices$).*".
         remove_duplicates (bool): Whether to remove duplicates within the new edges. Default is True.
         sort_indices (bool): Sort indices after adding edges. Default is True.
     """
@@ -54,7 +54,7 @@ class AddEdgeSelfLoops(GraphPreProcessorBase):
     Args:
         edge_indices (str): Name of indices in dictionary. Default is "edge_indices".
         edge_attributes (str): Name of related edge values or attributes.
-            This can be a match-string or list of names. Default is "^edge_.*".
+            This can be a match-string or list of names. Default is "^edge_(?!indices$).*".
         remove_duplicates (bool): Whether to remove duplicates. Default is True.
         sort_indices (bool): To sort indices after adding self-loops. Default is True.
         fill_value (in): The fill_value for all other edge properties.
@@ -87,7 +87,7 @@ class SortEdgeIndices(GraphPreProcessorBase):
     Args:
         edge_indices (str): Name of indices in dictionary. Default is "edge_indices".
         edge_attributes (str): Name of related edge values or attributes.
-            This can be a match-string or list of names. Default is "^edge_.*".
+            This can be a match-string or list of names. Default is "^edge_(?!indices$).*".
     """
 
     def __init__(self, *, edge_indices: str = "edge_indices", edge_attributes: str = "^edge_(?!indices$).*",
@@ -478,3 +478,53 @@ class AtomicChargesRepresentation(GraphPreProcessorBase):
         atom_scalars = np.expand_dims(oh, axis=-1) * np.expand_dims(charge_tensor, axis=1)  # (N, ohe, power)
         return atom_scalars.reshape((len(node_number), -1))
 
+
+class PrincipalMomentsOfInertia(GraphPreProcessorBase):
+    r"""Store the principle moments of the matrix of inertia for a set of node coordinates and masses into
+    a graph property defined by :obj:`graph_inertia`.
+
+    Args:
+        node_mass (str): Name of node mass in dictionary. Default is "node_mass".
+        node_coordinates (str): Name of node coordinates. Defaults to "node_coordinates".
+        graph_inertia (str): Name of output property to store moments. Default is "graph_inertia".
+        shift_center_of_mass (bool): Whether to shift to center of mass. Default is True.
+    """
+
+    def __init__(self, *, node_mass: str = "node_mass", node_coordinates: str = "node_coordinates",
+                 graph_inertia: str = "graph_inertia", name="principal_moments_of_inertia",
+                 shift_center_of_mass: bool = True, **kwargs):
+        super().__init__(name=name, **kwargs)
+        self._to_obtain.update({"node_mass": node_mass, "node_coordinates": node_coordinates})
+        self._to_assign = graph_inertia
+        self._call_kwargs = {"shift_center_of_mass": shift_center_of_mass}
+        self._config_kwargs.update({"node_mass": node_mass, "node_coordinates": node_coordinates,
+                                    "graph_inertia": graph_inertia, **self._call_kwargs})
+
+    def call(self, *, node_mass: np.ndarray, node_coordinates: np.ndarray, shift_center_of_mass: bool):
+        if node_mass is None or node_coordinates is None:
+            return None
+        return get_principal_moments_of_inertia(
+            masses=node_mass, coordinates=node_coordinates, shift_center_of_mass=shift_center_of_mass)
+
+
+class ShiftToUnitCell(GraphPreProcessorBase):
+    r"""Shift atomic coordinates into the Unit cell of a periodic lattice.
+
+    Args:
+        node_coordinates (str): Name of node coordinates. Defaults to "node_coordinates".
+        graph_lattice (str): Name of the graph lattice. Defaults to "graph_lattice".
+    """
+
+    def __init__(self, *, node_coordinates: str = "node_coordinates", graph_lattice: str = "graph_lattice",
+                 name="shift_to_unit_cell", **kwargs):
+        super().__init__(name=name, **kwargs)
+        self._to_obtain.update({"node_coordinates": node_coordinates, "graph_lattice": graph_lattice})
+        self._to_assign = node_coordinates
+        self._call_kwargs = {}
+        self._config_kwargs.update({"node_coordinates": node_coordinates, "graph_lattice": graph_lattice,
+                                    **self._call_kwargs})
+
+    def call(self, *, node_coordinates: np.ndarray, graph_lattice: np.ndarray, ):
+        if node_coordinates is None or graph_lattice is None:
+            return None
+        return shift_coordinates_to_unit_cell(coordinates=node_coordinates, lattice=graph_lattice)
\ No newline at end of file
