@@ -16,11 +16,12 @@ class DenseEmbedding(GraphBaseLayer):
     r"""Dense layer for ragged tensors representing a geometric or graph tensor such as node or edge embeddings.
     Latest tensorflow version now support ragged input for :obj:`ks.layers.Dense` with defined inner dimension.
     This layer is kept for backward compatibility but does not necessarily have to be used in models anymore.
-    A :obj:`DenseEmbedding` layer computes a densely-connected NN layer, i.e. a linear transformation of the input
+    A :obj:`Dense` layer computes a densely-connected NN layer, i.e. a linear transformation of the input
     :math:`\mathbf{x}` with the kernel weights matrix :math:`\mathbf{W}` and bias :math:`\mathbf{b}`
     plus (possibly non-linear) activation function :math:`\sigma`.
 
     .. math::
+
         \mathbf{x}' = \sigma (\mathbf{x} \mathbf{W} + \mathbf{b})
 
     """
@@ -86,10 +87,13 @@ class DenseEmbedding(GraphBaseLayer):
         return self._layer_dense(inputs, **kwargs)
 
 
-@ks.utils.register_keras_serializable(package='kgcnn', name='ActivationEmbedding')
+Dense = DenseEmbedding
+
+
+@ks.utils.register_keras_serializable(package="kgcnn", name="ActivationEmbedding")
 class ActivationEmbedding(GraphBaseLayer):
     r"""Activation layer for ragged tensors representing a geometric or graph tensor such as node or edge embeddings.
-    A :obj:`ActivationEmbedding` applies an activation function to an output, i.e. a transformation of the input
+    A :obj:`Activation` applies an activation function to an output, i.e. a transformation of the input
     :math:`\mathbf{x}` via activation function :math:`\sigma`.
 
     .. math::
@@ -121,7 +125,7 @@ class ActivationEmbedding(GraphBaseLayer):
         Returns:
             tf.RaggedTensor: Output tensor with activation applied.
         """
-        return self.call_on_values_tensor_of_ragged(self._layer_act, inputs, **kwargs)
+        return self.map_values(self._layer_act, inputs, **kwargs)
 
     def get_config(self):
         """Get config of layer."""
@@ -131,125 +135,10 @@ class ActivationEmbedding(GraphBaseLayer):
         return config
 
 
-@ks.utils.register_keras_serializable(package='kgcnn', name='LazyAdd')
-class LazyAdd(GraphBaseLayer):
-    r"""Layer that adds a list of inputs of e.g. geometric or graph tensor such as node or edge embeddings.
-    It takes as input a list of tensors, all the same shape, and returns a single tensor (also of the same shape).
-    For :obj:`RaggedTensor` the addition is directly performed on the `values` tensor of the ragged
-    input if all tensor in the list have `ragged_rank=1` and if `ragged_validate` is set to `False`.
-    Apart from debugging, this can imply a significant performance boost if ragged shape checks can be avoided.
-
-    .. math::
-        \mathbf{x}' = \sum_i \; \mathbf{x}_i
-    """
-
-    def __init__(self, **kwargs):
-        """Initialize layer."""
-        super(LazyAdd, self).__init__(**kwargs)
-        self._layer_add = ks.layers.Add()
-
-    def call(self, inputs, **kwargs):
-        r"""Forward pass corresponding to keras :obj:`Add` layer.
-
-        Args:
-            inputs (list): List of input tensor of same shape.
-
-        Returns:
-            tf.RaggedTensor: Single output tensor with same shape.
-        """
-        return self.call_on_values_tensor_of_ragged(self._layer_add, inputs, **kwargs)
+Activation = ActivationEmbedding
 
 
-@ks.utils.register_keras_serializable(package='kgcnn', name='LazySubtract')
-class LazySubtract(GraphBaseLayer):
-    r"""Layer that subtracts two inputs of e.g. geometric or graph tensor such as node or edge embeddings.
-    It takes as input a list of two tensors, both the same shape, and returns a single tensor (also of the same shape).
-    For :obj:`RaggedTensor` the subtraction is directly performed on the `values` tensor of the ragged
-    input if both tensor in the list have `ragged_rank=1` and if `ragged_validate` is set to `False`.
-    Apart from debugging, this can imply a significant performance boost if ragged shape checks can be avoided.
-
-    .. math::
-        \mathbf{x}' = \mathbf{x}_0 - \mathbf{x}_1
-    """
-
-    def __init__(self, **kwargs):
-        """Initialize layer."""
-        super(LazySubtract, self).__init__(**kwargs)
-        self._layer_subtract = ks.layers.Subtract()
-
-    def call(self, inputs, **kwargs):
-        r"""Forward pass corresponding to keras :obj:`Subtract` layer.
-
-        Args:
-            inputs (list): List of two input tensor of same shape.
-
-        Returns:
-            tf.RaggedTensor: Single output tensor which is (inputs[0] - inputs[1]).
-        """
-        return self.call_on_values_tensor_of_ragged(self._layer_subtract, inputs, **kwargs)
-
-
-@ks.utils.register_keras_serializable(package='kgcnn', name='LazyAverage')
-class LazyAverage(GraphBaseLayer):
-    r"""Layer that averages a list of inputs element-wise of e.g. geometric or graph tensor such as node or
-    edge embeddings.
-    It takes as input a list of tensors, all the same shape, and returns a single tensor (also of the same shape).
-    For :obj:`RaggedTensor` the average is directly performed on the `values` tensor of the ragged
-    input if all tensor in the list have `ragged_rank=1` and if `ragged_validate` is set to `False`.
-    Apart from debugging, this can imply a significant performance boost if ragged shape checks can be avoided.
-
-    .. math::
-        \mathbf{x}' = \frac{1}{N} \sum_{i=0,\dots,N} \;\; \mathbf{x}_i
-    """
-
-    def __init__(self, **kwargs):
-        """Initialize layer."""
-        super(LazyAverage, self).__init__(**kwargs)
-        self._layer_avg = ks.layers.Average()
-
-    def call(self, inputs, **kwargs):
-        r"""Forward pass corresponding to keras :obj:`Average` layer.
-
-        Args:
-            inputs (list): List of input tensor of same shape.
-
-        Returns:
-            tf.RaggedTensor: Single output tensor with same shape.
-        """
-        return self.call_on_values_tensor_of_ragged(self._layer_avg, inputs, **kwargs)
-
-
-@ks.utils.register_keras_serializable(package='kgcnn', name='LazyMultiply')
-class LazyMultiply(GraphBaseLayer):
-    r"""Layer that multiplies a list of inputs element-wise of e.g. geometric or graph tensor such as node or
-    edge embeddings.
-    It takes as input a list of tensors, all the same shape, and returns a single tensor (also of the same shape).
-    For :obj:`RaggedTensor` the multiplication is directly performed on the `values` tensor of the ragged
-    input if all tensor in the list have `ragged_rank=1` and if `ragged_validate` is set to `False`.
-    Apart from debugging, this can imply a significant performance boost if ragged shape checks can be avoided.
-
-    .. math::
-        \mathbf{x}' =  \Pi_i \; \mathbf{x}_i
-    """
-
-    def __init__(self, **kwargs):
-        """Initialize layer."""
-        super(LazyMultiply, self).__init__(**kwargs)
-        self._layer_mult = ks.layers.Multiply()
-
-    def call(self, inputs, **kwargs):
-        r"""Forward pass corresponding to keras :obj:`Multiply` layer.
-
-        Args:
-            inputs (list): List of input tensor of same shape.
-
-        Returns:
-            tf.RaggedTensor: Single output tensor with same shape.
-        """
-        return self.call_on_values_tensor_of_ragged(self._layer_mult, inputs, **kwargs)
-
-
-@ks.utils.register_keras_serializable(package='kgcnn', name='DropoutEmbedding')
+@ks.utils.register_keras_serializable(package="kgcnn", name="DropoutEmbedding")
 class DropoutEmbedding(GraphBaseLayer):
     r"""Layer that applies Dropout to the input of e.g. geometric or graph tensor such as node or
     edge embeddings.
@@ -288,7 +177,128 @@ class DropoutEmbedding(GraphBaseLayer):
         Returns:
             tf.RaggedTensor: Output tensor of same shape.
         """
-        return self.call_on_values_tensor_of_ragged(self._layer_drop, inputs, **kwargs)
+        return self.map_values(self._layer_drop, inputs, **kwargs)
+
+
+Dropout = DropoutEmbedding
+
+
+@ks.utils.register_keras_serializable(package='kgcnn', name='LazyAdd')
+class LazyAdd(GraphBaseLayer):
+    r"""Layer that adds a list of inputs of e.g. geometric or graph tensor such as node or edge embeddings.
+    It takes as input a list of tensors, all the same shape, and returns a single tensor (also of the same shape).
+    For :obj:`RaggedTensor` the addition is directly performed on the `values` tensor of the ragged
+    input if all tensor in the list have `ragged_rank=1` and if `ragged_validate` is set to `False`.
+    Apart from debugging, this can imply a significant performance boost if ragged shape checks can be avoided.
+
+    .. math::
+        \mathbf{x}' = \sum_i \; \mathbf{x}_i
+    """
+
+    def __init__(self, **kwargs):
+        """Initialize layer."""
+        super(LazyAdd, self).__init__(**kwargs)
+        self._layer_add = ks.layers.Add()
+
+    def call(self, inputs, **kwargs):
+        r"""Forward pass corresponding to keras :obj:`Add` layer.
+
+        Args:
+            inputs (list): List of input tensor of same shape.
+
+        Returns:
+            tf.RaggedTensor: Single output tensor with same shape.
+        """
+        return self.map_values(self._layer_add, inputs, **kwargs)
+
+
+@ks.utils.register_keras_serializable(package='kgcnn', name='LazySubtract')
+class LazySubtract(GraphBaseLayer):
+    r"""Layer that subtracts two inputs of e.g. geometric or graph tensor such as node or edge embeddings.
+    It takes as input a list of two tensors, both the same shape, and returns a single tensor (also of the same shape).
+    For :obj:`RaggedTensor` the subtraction is directly performed on the `values` tensor of the ragged
+    input if both tensor in the list have `ragged_rank=1` and if `ragged_validate` is set to `False`.
+    Apart from debugging, this can imply a significant performance boost if ragged shape checks can be avoided.
+
+    .. math::
+        \mathbf{x}' = \mathbf{x}_0 - \mathbf{x}_1
+    """
+
+    def __init__(self, **kwargs):
+        """Initialize layer."""
+        super(LazySubtract, self).__init__(**kwargs)
+        self._layer_subtract = ks.layers.Subtract()
+
+    def call(self, inputs, **kwargs):
+        r"""Forward pass corresponding to keras :obj:`Subtract` layer.
+
+        Args:
+            inputs (list): List of two input tensor of same shape.
+
+        Returns:
+            tf.RaggedTensor: Single output tensor which is (inputs[0] - inputs[1]).
+        """
+        return self.map_values(self._layer_subtract, inputs, **kwargs)
+
+
+@ks.utils.register_keras_serializable(package='kgcnn', name='LazyAverage')
+class LazyAverage(GraphBaseLayer):
+    r"""Layer that averages a list of inputs element-wise of e.g. geometric or graph tensor such as node or
+    edge embeddings.
+    It takes as input a list of tensors, all the same shape, and returns a single tensor (also of the same shape).
+    For :obj:`RaggedTensor` the average is directly performed on the `values` tensor of the ragged
+    input if all tensor in the list have `ragged_rank=1` and if `ragged_validate` is set to `False`.
+    Apart from debugging, this can imply a significant performance boost if ragged shape checks can be avoided.
+
+    .. math::
+        \mathbf{x}' = \frac{1}{N} \sum_{i=0,\dots,N} \;\; \mathbf{x}_i
+    """
+
+    def __init__(self, **kwargs):
+        """Initialize layer."""
+        super(LazyAverage, self).__init__(**kwargs)
+        self._layer_avg = ks.layers.Average()
+
+    def call(self, inputs, **kwargs):
+        r"""Forward pass corresponding to keras :obj:`Average` layer.
+
+        Args:
+            inputs (list): List of input tensor of same shape.
+
+        Returns:
+            tf.RaggedTensor: Single output tensor with same shape.
+        """
+        return self.map_values(self._layer_avg, inputs, **kwargs)
+
+
+@ks.utils.register_keras_serializable(package='kgcnn', name='LazyMultiply')
+class LazyMultiply(GraphBaseLayer):
+    r"""Layer that multiplies a list of inputs element-wise of e.g. geometric or graph tensor such as node or
+    edge embeddings.
+    It takes as input a list of tensors, all the same shape, and returns a single tensor (also of the same shape).
+    For :obj:`RaggedTensor` the multiplication is directly performed on the `values` tensor of the ragged
+    input if all tensor in the list have `ragged_rank=1` and if `ragged_validate` is set to `False`.
+    Apart from debugging, this can imply a significant performance boost if ragged shape checks can be avoided.
+
+    .. math::
+        \mathbf{x}' =  \Pi_i \; \mathbf{x}_i
+    """
+
+    def __init__(self, **kwargs):
+        """Initialize layer."""
+        super(LazyMultiply, self).__init__(**kwargs)
+        self._layer_mult = ks.layers.Multiply()
+
+    def call(self, inputs, **kwargs):
+        r"""Forward pass corresponding to keras :obj:`Multiply` layer.
+
+        Args:
+            inputs (list): List of input tensor of same shape.
+
+        Returns:
+            tf.RaggedTensor: Single output tensor with same shape.
+        """
+        return self.map_values(self._layer_mult, inputs, **kwargs)
 
 
 @ks.utils.register_keras_serializable(package='kgcnn', name='LazyConcatenate')
@@ -345,7 +355,7 @@ class LazyConcatenate(GraphBaseLayer):
         Returns:
             tf.tensor: Single concatenated tensor.
         """
-        return self.call_on_values_tensor_of_ragged(tf.concat, inputs, axis=self.axis)
+        return self.map_values(tf.concat, inputs, axis=self.axis)
 
     def get_config(self):
         """Get config of layer."""
@@ -361,7 +371,6 @@ class ExpandDims(GraphBaseLayer):
     For :obj:`RaggedTensor` the expansion via :obj:`tf.expand_dims` is directly performed on the `values` tensor
     of the ragged input if it has `ragged_rank=1` and if `ragged_validate` is set to `False` and
     axis is larger than the ragged axis, which requires `axis>1`.
-
     """
 
     def __init__(self,
@@ -398,7 +407,7 @@ class ExpandDims(GraphBaseLayer):
         Returns:
             tf.tensor: Single tensor with inserted dimension.
         """
-        return self.call_on_values_tensor_of_ragged(tf.expand_dims, inputs, axis=self.axis)
+        return self.map_values(tf.expand_dims, inputs, axis=self.axis)
 
     def get_config(self):
         """Get config of layer."""
@@ -434,7 +443,7 @@ class ZerosLike(GraphBaseLayer):
         Returns:
             tf.RaggedTensor: Zero-like tensor of input.
         """
-        return self.call_on_values_tensor_of_ragged(tf.zeros_like, inputs)
+        return self.map_values(tf.zeros_like, inputs)
 
 
 @ks.utils.register_keras_serializable(package='kgcnn', name='OptionalInputEmbedding')
