@@ -1,21 +1,42 @@
-from typing import List, Tuple, Dict, Optional, Any
+import typing as t
 
 import numpy as np
 import tensorflow as tf
 
 from tensorflow.python.keras.engine import compile_utils
 from kgcnn.layers.base import GraphBaseLayer
-from kgcnn.layers.modules import DenseEmbedding, OptionalInputEmbedding
-from kgcnn.layers.modules import ActivationEmbedding, DropoutEmbedding
+from kgcnn.layers.modules import Dense, OptionalInputEmbedding
+from kgcnn.layers.modules import Activation, Dropout
 from kgcnn.layers.modules import LazyConcatenate, LazyAverage
 from kgcnn.layers.conv.gat_conv import MultiHeadGATV2Layer
 from kgcnn.layers.pooling import PoolingLocalEdges
 from kgcnn.layers.pooling import PoolingWeightedNodes, PoolingNodes
+from kgcnn.xai.base import ImportanceExplanationMixin
 
 ks = tf.keras
 
 
-class MEGAN(ks.models.Model):
+def shifted_sigmoid(x, shift=5, multiplier=1):
+    return ks.backend.sigmoid((x - shift) / multiplier)
+
+
+class ExplanationSparsityRegularization(GraphBaseLayer):
+
+    def __init__(self,
+                 factor: float = 1.0,
+                 **kwargs):
+        super(ExplanationSparsityRegularization, self).__init__(**kwargs)
+        self.factor = factor
+
+    def call(self, inputs):
+        # importances: ([batch], [N], K)
+        importances = inputs
+
+        loss = tf.reduce_mean(tf.math.abs(importances))
+        self.add_loss(loss * self.factor)
+
+
+class MEGAN(ks.models.Model, ImportanceExplanationMixin):
     """
     MEGAN: Multi Explanation Graph Attention Network
 
@@ -28,14 +49,14 @@ class MEGAN(ks.models.Model):
 
     def __init__(self,
                  # convolutional network related arguments
-                 units: List[int],
+                 units: t.List[int],
                  activation: str = "kgcnn>leaky_relu",
                  use_bias: bool = True,
                  dropout_rate: float = 0.0,
                  use_edge_features: bool = True,
                  input_embedding: dict = None,
                  # node/edge importance related arguments
-                 importance_units: List[int] = [],
+                 importance_units: t.List[int] = [],
                  importance_channels: int = 2,
                  importance_activation: str = "sigmoid",  # do not change
                  importance_dropout_rate: float = 0.0,  # do not change
@@ -44,17 +65,16 @@ class MEGAN(ks.models.Model):
                  sparsity_factor: float = 0.0,
                  concat_heads: bool = True,
                  # mlp tail end related arguments
-                 final_units: List[int] = [1],
+                 final_units: t.List[int] = [1],
                  final_dropout_rate: float = 0.0,
                  final_activation: str = 'linear',
                  final_pooling: str = 'sum',
-                 regression_limits: Optional[Tuple[float, float]] = None,
-                 regression_reference: Optional[float] = None,
+                 regression_limits: t.Optional[t.Tuple[float, float]] = None,
+                 regression_reference: t.Optional[float] = None,
                  return_importances: bool = True,
                  **kwargs):
         """
         Args:
-
             units: A list of ints where each element configures an additional attention layer. The numeric
                 value determines the number of hidden units to be used in the attention heads of that layer
             activation: The activation function to be used within the attention layers of the network
@@ -125,7 +145,7 @@ class MEGAN(ks.models.Model):
         # ~ MAIN CONVOLUTIONAL / ATTENTION LAYERS
         if self.input_embedding:
             self.embedding_nodes = OptionalInputEmbedding(**self.input_embedding["node"])
-        self.attention_layers: List[GraphBaseLayer] = []
+        self.attention_layers: t.List[GraphBaseLayer] = []
         for u in self.units:
             lay = MultiHeadGATV2Layer(
                 units=u,
@@ -138,10 +158,10 @@ class MEGAN(ks.models.Model):
             )
             self.attention_layers.append(lay)
 
-        self.lay_dropout = DropoutEmbedding(rate=self.dropout_rate)
+        self.lay_dropout = Dropout(rate=self.dropout_rate)
 
         # ~ EDGE IMPORTANCES
-        self.lay_act_importance = ActivationEmbedding(activation=self.importance_activation)
+        self.lay_act_importance = Activation(activation=self.importance_activation)
         self.lay_concat_alphas = LazyConcatenate(axis=-1)
 
         self.lay_pool_edges_in = PoolingLocalEdges(pooling_method='mean', pooling_index=0)
@@ -153,17 +173,19 @@ class MEGAN(ks.models.Model):
         self.node_importance_acts = ['relu' for _ in importance_units] + ['linear']
         self.node_importance_layers = []
         for u, act in zip(self.node_importance_units, self.node_importance_acts):
-            lay = DenseEmbedding(
+            lay = Dense(
                 units=u,
                 activation=act,
                 use_bias=use_bias
             )
             self.node_importance_layers.append(lay)
+            
+        self.lay_sparsity = ExplanationSparsityRegularization(factor=self.sparsity_factor)
 
         # ~ OUTPUT / MLP TAIL END
         self.lay_pool_out = PoolingNodes(pooling_method=self.final_pooling)
         self.lay_concat_out = LazyConcatenate(axis=-1)
-        self.lay_final_dropout = DropoutEmbedding(rate=self.final_dropout_rate)
+        self.lay_final_dropout = Dropout(rate=self.final_dropout_rate)
 
         self.final_acts = ["relu" for _ in self.final_units]
         self.final_acts[-1] = self.final_activation
@@ -171,7 +193,7 @@ class MEGAN(ks.models.Model):
         self.final_biases[-1] = False
         self.final_layers = []
         for u, act, bias in zip(self.final_units, self.final_acts, self.final_biases):
-            lay = DenseEmbedding(
+            lay = Dense(
                 units=u,
                 activation=act,
                 use_bias=use_bias
@@ -180,10 +202,11 @@ class MEGAN(ks.models.Model):
 
         # ~ EXPLANATION ONLY TRAIN STEP
         self.bce_loss = ks.losses.BinaryCrossentropy()
-        self.compiled_bce_loss = compile_utils.LossesContainer(self.bce_loss)
+        self.compiled_classification_loss = compile_utils.LossesContainer(self.bce_loss)
 
         self.mse_loss = ks.losses.MeanSquaredError()
-        self.compiled_mse_loss = compile_utils.LossesContainer(self.mse_loss)
+        self.mae_loss = ks.losses.MeanAbsoluteError()
+        self.compiled_regression_loss = compile_utils.LossesContainer(self.mae_loss)
 
         if self.regression_limits is not None:
             self.regression_width = np.abs(self.regression_limits[1] - self.regression_limits[0])
@@ -264,6 +287,7 @@ class MEGAN(ks.models.Model):
         node_importances_tilde = self.lay_act_importance(node_importances_tilde)
 
         node_importances = node_importances_tilde * pooled_edges
+        self.lay_sparsity(node_importances)
 
         # Here we apply the global pooling. It is important to note that we do K separate pooling operations
         # were each time we use the same node embeddings x but a different slice of the node importances as
@@ -285,6 +309,10 @@ class MEGAN(ks.models.Model):
             if training:
                 self.lay_final_dropout(out, training=training)
 
+        if self.doing_regression:
+            reference = tf.ones_like(out) * self.regression_reference
+            out = out + reference
+
         # Usually, the node and edge importance tensors would be direct outputs of the model as well, but
         # we need the option to just return the output alone to be compatible with the standard model
         # evaluation pipeline already implemented in the library.
@@ -294,10 +322,19 @@ class MEGAN(ks.models.Model):
             return out
 
     def regression_augmentation(self,
-                                out_true):
+                                out_true: tf.RaggedTensor):
+        """
+        Given the tensor ([B], 1) of true regression target values, this method will return two derived
+        tensors: The first one is a ([B], 2) tensor of normalized distances of the corresponding true
+        values to ``self.regression_reference`` and the second is a ([B], 2) boolean mask tensor.
+        Args:
+            out_true: A tensor of shape ([B], 1) of the true target values of the current batch.
+        Returns:
+            A tuple of two tensors each with the shape ([B], 2)
+        """
         center_distances = tf.abs(out_true - self.regression_reference)
-        center_distances = (center_distances * self.importance_multiplier) / (0.5 * self.regression_width)
-        center_distances = tf.expand_dims(center_distances, axis=-1)
+        center_distances = (center_distances * self.importance_multiplier) / (
+                0.5 * self.regression_width)
 
         # So we need two things: a "samples" tensor and a "mask" tensor. We are going to use the samples
         # tensor as the actual ground truth which acts as the regression target during the explanation
@@ -306,99 +343,89 @@ class MEGAN(ks.models.Model):
 
         # The "lower" part is all the samples which have a target value below the reference value.
         lo_mask = tf.where(out_true < self.regression_reference, 1.0, 0.0)
-        lo_mask = tf.expand_dims(lo_mask, axis=-1)
-
         # The "higher" part all of the samples above reference
         hi_mask = tf.where(out_true > self.regression_reference, 1.0, 0.0)
-        hi_mask = tf.expand_dims(hi_mask, axis=-1)
 
         samples = tf.concat([center_distances, center_distances], axis=-1)
         mask = tf.concat([lo_mask, hi_mask], axis=-1)
+
         return samples, mask
 
-    def train_step_explanation(self, x, y):
-
-        if self.return_importances:
-            out_true, _, _ = y
-        else:
-            out_true = y
-
-        exp_loss = 0
-        with tf.GradientTape() as tape:
-
-            y_pred = self(x, training=True, return_importances=True)
-            out_pred, ni_pred, ei_pred = y_pred
-
-            # First of all we need to assemble the approximated model output, which is simply calculated
-            # by applying a global pooling operation on the corresponding slice of the node importances.
-            # So for each slice (each importance channel) we get a single value, which we then concatenate
-            # into an output vector with K dimensions.
-            outs = []
-            for k in range(self.importance_channels):
-                node_importances_slice = tf.expand_dims(ni_pred[:, :, k], axis=-1)
-                out = self.lay_pool_out(node_importances_slice)
-
-                outs.append(out)
-
-            # outs: ([batch], K)
-            outs = self.lay_concat_out(outs)
-
-            if self.doing_regression:
-                out_true, mask = self.regression_augmentation(out_true)
-                out_pred = outs
-                exp_loss = self.compiled_mse_loss(out_true * mask, out_pred * mask)
-
-            else:
-                out_pred = ks.backend.sigmoid(outs)
-                exp_loss = self.compiled_bce_loss(out_true, out_pred)
-
-            exp_loss *= self.importance_factor
-
-        # Compute gradients
-        trainable_vars = self.trainable_variables
-        exp_gradients = tape.gradient(exp_loss, trainable_vars)
-
-        # Update weights
-        self.optimizer.apply_gradients(zip(exp_gradients, trainable_vars))
-
-        return {'exp_loss': exp_loss}
-
     def train_step(self, data):
+        """
+        The method which is called for each individual batch of the training step. In this method a forward
+        pass of the input data of the batch is performed, the loss is calculated, the gradients of this loss
+        w.r.t to the model parameters are calculated and these gradients are used to update the weights of
+        the model.
+        This custom train step implements the "explanation co-training": Additionally to the main prediction
+        loss there is also a loss term which uses only the node importance values to approximate a
+        normalized versions of the target values.
+        Args:
+            data: Tuple of two tensors (X, Y) where X is the batch tensor of all the inputs and Y is the
+                batch tensor of the target output values.
+        Returns:
+            A dictionary whose keys are the names of metrics and the values are the corresponding values
+            of these metrics for this iteration of the training.
+        """
         if len(data) == 3:
             x, y, sample_weight = data
         else:
             sample_weight = None
             x, y = data
 
-        # The "train_step_explanation" method will execute the entire explanation train step once. This
-        # mainly involves creating an approximate solution of the original regression / classification
-        # problem using ONLY the node importances of the different channels and then performing one
-        # complete weight update based on the corresponding loss.
-        if self.importance_factor != 0:
-            exp_metrics = self.train_step_explanation(x, y)
-        else:
-            exp_metrics = {}
-
         with tf.GradientTape() as tape:
-            y_pred = self(x, training=True)
             if self.return_importances:
                 out_true, ni_true, ei_true = y
-                out_pred, ni_pred, ei_pred = self(x, training=True)
-                loss = self.compiled_loss(
-                    [out_true, ni_true, ei_true],
-                    [out_pred, ni_pred, ei_pred],
-                    sample_weight=sample_weight,
-                    regularization_losses=self.losses,
-                )
             else:
                 out_true = y
-                out_pred = self(x, training=True)
-                loss = self.compiled_loss(
-                    out_true,
-                    out_pred,
-                    sample_weight=sample_weight,
-                    regularization_losses=self.losses,
-                )
+
+            y_pred = self(x, training=True, return_importances=True)
+            out_pred, ni_pred, ei_pred = y_pred
+            loss = self.compiled_loss(
+                y,
+                y_pred if self.return_importances else out_pred,
+                sample_weight=sample_weight,
+                regularization_losses=self.losses,
+            )
+
+            exp_metrics = {}
+            if self.importance_factor != 0:
+                # ~ explanation loss
+                # First of all we need to assemble the approximated model output, which is simply calculated
+                # by applying a global pooling operation on the corresponding slice of the node importances.
+                # So for each slice (each importance channel) we get a single value, which we then
+                # concatenate into an output vector with K dimensions.
+                outs = []
+                for k in range(self.importance_channels):
+                    node_importances_slice = tf.expand_dims(ni_pred[:, :, k], axis=-1)
+                    out = self.lay_pool_out(node_importances_slice)
+
+                    outs.append(out)
+
+                # outs: ([batch], K)
+                outs = self.lay_concat_out(outs)
+
+                if self.doing_regression:
+                    out_true, mask = self.regression_augmentation(out_true)
+                    out_pred = outs
+                    exp_loss = self.importance_channels * self.compiled_regression_loss(out_true * mask,
+                                                                                        out_pred * mask)
+
+                else:
+                    out_pred = shifted_sigmoid(
+                        outs,
+
+                        # shift=self.importance_multiplier,
+                        # multiplier=(self.importance_multiplier / 5)
+
+                        shift=self.importance_multiplier,
+                        multiplier=1,
+                    )
+                    exp_loss = self.compiled_classification_loss(out_true, out_pred)
+
+                exp_loss *= self.importance_factor
+                exp_metrics['exp_loss'] = exp_loss
+                loss += exp_loss
 
         trainable_vars = self.trainable_variables
         gradients = tape.gradient(loss, trainable_vars)
@@ -407,7 +434,7 @@ class MEGAN(ks.models.Model):
 
         self.compiled_metrics.update_state(
             y,
-            y_pred,
+            out_pred if not self.return_importances else [out_pred, ni_pred, ei_pred],
             sample_weight=sample_weight
         )
 
@@ -418,10 +445,26 @@ class MEGAN(ks.models.Model):
             **exp_metrics
         }
 
+    def explain_importances(self,
+                            x: t.Sequence[tf.Tensor],
+                            **kwargs
+                            ) -> t.Tuple[tf.RaggedTensor, tf.RaggedTensor]:
+        y, node_importances, edge_importances = self(x, return_importances=True)
+        return node_importances, edge_importances
 
-def make_model(inputs: Optional[list] = None,
+
+def make_model(inputs: t.Optional[list] = None,
                **kwargs
                ):
+    r"""Functional model definition of MEGAN. Please check documentation of :obj:`kgcnn.literature.MEGAN` .
+
+    Args:
+        inputs (list): List of dictionaries unpacked in :obj:`tf.keras.layers.Input`. Order must match model definition.
+        kwargs: Kwargs for MEGAN model. Please check documentation of :obj:`kgcnn.literature.MEGAN` .
+
+    Returns:
+        :obj:`tf.keras.models.Model`
+    """
     # Building the actual model
     megan = MEGAN(**kwargs)
 
@@ -430,4 +473,5 @@ def make_model(inputs: Optional[list] = None,
     layer_inputs = [ks.layers.Input(**kwargs) for kwargs in inputs]
     outputs = megan(layer_inputs)
     model = ks.models.Model(inputs=layer_inputs, outputs=outputs)
-    return model
+
+    return megan
