@@ -2,11 +2,11 @@ import tensorflow as tf
 from kgcnn.layers.casting import ChangeTensorType
 from kgcnn.layers.conv.mpnn_conv import GRUUpdate, TrafoEdgeNetMessages, MatMulMessages
 from kgcnn.layers.gather import GatherNodesOutgoing, GatherNodesIngoing
-from kgcnn.layers.modules import DenseEmbedding, LazyConcatenate, OptionalInputEmbedding
+from kgcnn.layers.modules import Dense, LazyConcatenate, OptionalInputEmbedding
 from kgcnn.layers.mlp import GraphMLP, MLP
 from kgcnn.layers.pooling import PoolingLocalEdges, PoolingNodes
 from kgcnn.layers.pool.set2set import PoolingSet2Set
-from kgcnn.utils.models import update_model_kwargs
+from kgcnn.model.utils import update_model_kwargs
 from kgcnn.layers.geom import NodePosition, NodeDistanceEuclidean, GaussBasisLayer, ShiftPeriodicLattice
 
 ks = tf.keras
@@ -124,7 +124,7 @@ def make_model(inputs: list = None,
         ed = GaussBasisLayer(**gauss_args)(ed)
 
     # Make hidden dimension
-    n = DenseEmbedding(node_dim, activation="linear")(n0)
+    n = Dense(node_dim, activation="linear")(n0)
 
     # Make edge networks.
     edge_net_in = GraphMLP(**edge_mlp)(ed)
@@ -150,7 +150,7 @@ def make_model(inputs: list = None,
     if output_embedding == 'graph':
         if use_set2set:
             # output
-            out = DenseEmbedding(set2set_args['channels'], activation="linear")(n)
+            out = Dense(set2set_args['channels'], activation="linear")(n)
             out = PoolingSet2Set(**set2set_args)(out)
         else:
             out = PoolingNodes(**pooling_args)(n)
@@ -283,7 +283,7 @@ def make_crystal_model(inputs: list = None,
         ed = GaussBasisLayer(**gauss_args)(ed)
 
     # Make hidden dimension
-    n = DenseEmbedding(node_dim, activation="linear")(n0)
+    n = Dense(node_dim, activation="linear")(n0)
 
     # Make edge networks.
     edge_net_in = GraphMLP(**edge_mlp)(ed)
@@ -309,7 +309,7 @@ def make_crystal_model(inputs: list = None,
     if output_embedding == 'graph':
         if use_set2set:
             # output
-            out = DenseEmbedding(set2set_args['channels'], activation="linear")(n)
+            out = Dense(set2set_args['channels'], activation="linear")(n)
             out = PoolingSet2Set(**set2set_args)(out)
         else:
             out = PoolingNodes(**pooling_args)(n)
