@@ -7,10 +7,11 @@ module_logger.setLevel(logging.INFO)
 
 
 class MolGraphInterface:
-    r"""The `MolGraphInterface` defines the base class interface to handle a molecular graph. The method implementation
-    to generate a mol-instance from smiles etc. can be obtained from different backends like `RDkit`. The mol-instance
-    of a chemical informatics package like `RDkit` is treated via composition. The interface is designed to
-    extract a graph from a mol instance not to make a mol object from a graph.
+    r"""The `MolGraphInterface` defines the base class interface to handle a molecular graph.
+
+    The method implementation to generate a molecule-instance from smiles etc. can be obtained from different backends
+    like `RDkit`. The mol-instance of a chemical informatics package like `RDkit` is treated via composition.
+    The interface is designed to extract a graph from a mol instance, not to make a mol object from a graph.
 
     """
 
@@ -25,11 +26,11 @@ class MolGraphInterface:
         self.mol = mol
         self._make_directed = make_directed
 
-    def add_hs(self):
+    def add_hs(self, **kwargs):
         """Add hydrogen to molecule instance."""
         raise NotImplementedError("Method for `MolGraphInterface` must be implemented in sub-class.")
 
-    def remove_hs(self):
+    def remove_hs(self, **kwargs):
         """Remove hydrogen from molecule instance."""
         raise NotImplementedError("Method for `MolGraphInterface` must be implemented in sub-class.")
 
@@ -52,7 +53,7 @@ class MolGraphInterface:
         """
         raise NotImplementedError("Method for `MolGraphInterface` must be implemented in sub-class.")
 
-    def to_smiles(self):
+    def to_smiles(self, **kwargs):
         """Return a smile string representation of the mol instance.
 
         Returns:
@@ -72,7 +73,7 @@ class MolGraphInterface:
         """
         raise NotImplementedError("Method for `MolGraphInterface` must be implemented in sub-class.")
 
-    def to_mol_block(self):
+    def to_mol_block(self, **kwargs):
         """Make a more extensive string representation containing coordinates and bond information from self.
 
         Returns:
@@ -80,6 +81,12 @@ class MolGraphInterface:
         """
         raise NotImplementedError("Method for `MolGraphInterface` must be implemented in sub-class.")
 
+    def clean(self, **kwargs):
+        raise NotImplementedError("Method for `MolGraphInterface` must be implemented in sub-class.")
+
+    def compute_charges(self, method="gasteiger", **kwargs):
+        raise NotImplementedError("Method for `MolGraphInterface` must be implemented in sub-class.")
+
     @property
     def node_number(self):
         """Return list of node numbers which is the atomic number of atoms in the molecule"""
