@@ -9,7 +9,7 @@ from kgcnn.data.utils import save_json_file, load_json_file
 
 class ExtensiveMolecularScaler:
     r"""Scaler for extensive properties like energy to remove a simple linear behaviour with additive atom
-    contributions. Interface is designed after scikit-learn standard scaler. Internally Ridge regression ist used.
+    contributions. Interface is designed after scikit-learn scaler. Internally Ridge regression ist used.
     Only the atomic number is used as extensive scaler. This could be further improved by also taking bonds and
     interactions into account, e.g. as energy contribution.
 
@@ -32,23 +32,25 @@ class ExtensiveMolecularScaler:
         new_scaler = ExtensiveMolecularScaler()
         new_scaler.load("example.json")
         print(new_scaler.inverse_transform(scaler.transform(X=data, atomic_number=mol_num), atomic_number=mol_num))
+
     """
 
     _attributes_list_sklearn = ["n_features_in_", "coef_", "intercept_", "n_iter_", "feature_names_in_"]
     _attributes_list_mol = ["scale_", "_fit_atom_selection", "_fit_atom_selection_mask"]
     max_atomic_number = 95
 
-    def __init__(self, alpha: float = 1e-9, fit_intercept: bool = False, **kwargs):
+    def __init__(self, alpha: float = 1e-9, fit_intercept: bool = False, standardize_scale: bool = True, **kwargs):
         r"""Initialize scaler with parameters directly passed to scikit-learns :obj:`Ridge()`.
 
         Args:
             alpha (float): Regularization parameter for regression.
             fit_intercept (bool): Whether to allow a constant offset per target.
+            standardize_scale (bool): Whether to standardize output after offset removal.
             kwargs: Additional arguments passed to :obj:`Ridge()`.
         """
 
         self.ridge = Ridge(alpha=alpha, fit_intercept=fit_intercept, **kwargs)
-
+        self._standardize_scale = standardize_scale
         self._fit_atom_selection_mask = None
         self._fit_atom_selection = None
         self.scale_ = None
@@ -89,7 +91,10 @@ class ExtensiveMolecularScaler:
         total_number = np.array(total_number)
         self.ridge.fit(total_number, molecular_property, sample_weight=sample_weight)
         diff = molecular_property - self.ridge.predict(total_number)
-        self.scale_ = np.std(diff, axis=0)
+        if self._standardize_scale:
+            self.scale_ = np.std(diff, axis=0)
+        else:
+            self.scale_ = np.ones(diff.shape[1:], dtype="float")
         return self
 
     def predict(self, atomic_number):
@@ -136,26 +141,35 @@ class ExtensiveMolecularScaler:
         plt.legend(loc='upper left', fontsize='x-small')
         plt.show()
 
-    def transform(self, X, *, copy=None, atomic_number):
+    def transform(self, X, *, y=None, copy=True, atomic_number):
         """Transform any atomic number list with matching properties based on previous fit. Also std-scaled.
 
         Args:
             X (np.ndarray): Array of atomic properties of shape `(n_samples, n_properties)`.
-            copy (bool): Not yet implemented.
+            y (np.ndarray): Ignored.
+            copy (bool): Whether to copy or change in place.
             atomic_number (list): List of array of atomic numbers. Example [np.array([7,1,1,1]), ...].
 
         Returns:
             np.ndarray: Transformed atomic properties fitted. Shape is `(n_samples, n_properties)`.
         """
-        return (X - self.predict(atomic_number)) / np.expand_dims(self.scale_, axis=0)
+        if copy:
+            X = X - self.predict(atomic_number)
+            if self._standardize_scale:
+                X = X / np.expand_dims(self.scale_, axis=0)
+        else:
+            X -= self.predict(atomic_number)
+            if self._standardize_scale:
+                X /= np.expand_dims(self.scale_, axis=0)
+        return X
 
-    def fit_transform(self, X, *, y=None, copy=None, sample_weight=None, atomic_number=None):
+    def fit_transform(self, X, *, y=None, copy=True, sample_weight=None, atomic_number=None):
         """Combine fit and transform methods in one call.
 
         Args:
             X (np.ndarray): Array of atomic properties of shape `(n_samples, n_properties)`.
             y (np.ndarray): Ignored.
-            copy (bool): Not yet implemented.
+            copy (bool): Whether to copy or change in place.
             atomic_number (list): List of array of atomic numbers. Example [np.array([7,1,1,1]), ...].
             sample_weight: Sample weights `(n_samples,)` directly passed to :obj:`Ridge()`. Default is None.
 
@@ -167,22 +181,34 @@ class ExtensiveMolecularScaler:
         self.fit(X=X, y=y, atomic_number=atomic_number, sample_weight=sample_weight)
         return self.transform(X=X, copy=copy, atomic_number=atomic_number)
 
-    def inverse_transform(self, X, *, copy=None, atomic_number):
+    def inverse_transform(self, X, *, y=None, copy=True, atomic_number):
         """Reverse the transform method to original properties without offset and scaled to original units.
 
         Args:
             X (np.ndarray): Array of atomic properties of shape `(n_samples, n_properties)`
-            copy (bool): Not yet implemented.
+            y (np.ndarray): Ignored.
+            copy (bool): Whether to copy or change in place.
             atomic_number (list): List of array of atomic numbers. Example [np.array([7,1,1,1]), ...].
 
         Returns:
             np.ndarray: Original atomic properties. Shape is `(n_samples, n_properties)`.
         """
-        return X * np.expand_dims(self.scale_, axis=0) + self.predict(atomic_number)
+        if copy:
+            if self._standardize_scale:
+                X = X * np.expand_dims(self.scale_, axis=0)
+            X = X + self.predict(atomic_number)
+        else:
+            if self._standardize_scale:
+                X *= np.expand_dims(self.scale_, axis=0)
+            X += self.predict(atomic_number)
+        return X
 
     def get_config(self):
         """Get configuration for scaler."""
-        return self.ridge.get_params()
+        config = {}
+        config.update(self.ridge.get_params())
+        config.update({"standardize_scale": self._standardize_scale})
+        return config
 
     def set_config(self, config):
         """Set configuration for scaler.
@@ -190,7 +216,10 @@ class ExtensiveMolecularScaler:
         Args:
             config (dict): Config dictionary.
         """
-        return self.ridge.set_params(**config)
+        self._standardize_scale = config["standardize_scale"]
+        config_ridge = {key: value for key, value in config.items() if key not in ["standardize_scale"]}
+        self.ridge.set_params(**config_ridge)
+        return self
 
     def get_weights(self) -> dict:
         """Get weights for this scaler after fit."""
@@ -259,16 +288,21 @@ class ExtensiveMolecularScaler:
         # Could verify class_name and module_name here.
         self.set_config(full_info["config"])
         self.set_weights(full_info["weights"])
+        return self
 
 
 class QMGraphLabelScaler:
     r"""A scaler that scales QM targets differently. For now, the main difference is that intensive and extensive
     properties are scaled differently. In principle, also dipole, polarizability or rotational constants
-    could to be standardized differently.
+    could to be standardized differently. Interface is designed after scikit-learn scaler.
 
     The class is simply a list of separate scaler and scales each target of shape [N_samples, target] with a scaler
     from its list. :obj:`QMGraphLabelScaler` is intended as a scaler list class.
 
+    The scaler uses `y` argument but for compatibility reason will also work with labels assigned to `X`
+    argument in :obj:`fit`, :obj:`transform` etc. Each label is passed to the corresponding scaler in list simply
+    as first argument without keyword `X` or `y`.
+
     .. code-block:: python
 
         import numpy as np
@@ -279,15 +313,16 @@ class QMGraphLabelScaler:
             np.array([6, 6, 1, 1, 1, 1]), np.array([6, 6, 1, 1]), np.array([6, 6, 1, 1, 1, 1, 1, 1])
         ]
         scaler = QMGraphLabelScaler([ExtensiveMolecularScaler(), StandardScaler()])
-        scaler.fit(X=data, atomic_number=mol_num)
+        scaler.fit(y=data, atomic_number=mol_num)
         print(scaler.get_weights())
         print(scaler.get_config())
-        print(scaler.inverse_transform(scaler.transform(X=data, atomic_number=mol_num), atomic_number=mol_num))
+        print(scaler.inverse_transform(scaler.transform(y=data, atomic_number=mol_num), atomic_number=mol_num))
         print(data)
         scaler.save("example.json")
         new_scaler = QMGraphLabelScaler([ExtensiveMolecularScaler(), StandardScaler()])
         new_scaler.load("example.json")
-        print(new_scaler.inverse_transform(scaler.transform(X=data, atomic_number=mol_num), atomic_number=mol_num))
+        print(new_scaler.inverse_transform(scaler.transform(y=data, atomic_number=mol_num), atomic_number=mol_num))
+
     """
 
     def __init__(self, scaler: list):
@@ -304,9 +339,9 @@ class QMGraphLabelScaler:
 
             # Otherwise, must be serialized version of a scaler.
             if not isinstance(x, dict):
-                raise TypeError("Single scaler for `QMGraphLabelScaler` must be dict, got '%s'." % x)
+                raise TypeError("Single scaler for `QMGraphLabelScaler` deserialization must be dict, got '%s'." % x)
             if "class_name" not in x:
-                raise ValueError("Scaler class for single target must be defined, got '%s'" % x)
+                raise ValueError("Scaler class for single target must be defined, got '%s'." % x)
 
             # Pick allowed scaler.
             if x["class_name"] == "StandardScaler":
@@ -314,89 +349,100 @@ class QMGraphLabelScaler:
             elif x["class_name"] == "ExtensiveMolecularScaler":
                 self.scaler_list.append(ExtensiveMolecularScaler(**x["config"]))
             else:
-                raise ValueError("Unsupported scaler %s" % x["name"])
+                raise ValueError("Unsupported scaler '%s'." % x["name"])
 
-    def fit_transform(self, X, *, y=None, copy=None, sample_weight=None, atomic_number=None):
-        r"""Fit and transform all target labels for QM9.
+    def fit_transform(self, y=None, *, X=None, copy=True, sample_weight=None, atomic_number=None):
+        r"""Fit and transform all target labels for QM.
 
         Args:
-            X (np.ndarray): Array of atomic properties of shape `(n_samples, n_properties)`.
-            y (np.ndarray): Not used.
-            copy (bool): Not fully supported.
+            y (np.ndarray): Array of atomic labels of shape `(n_samples, n_labels)`.
+            X (np.ndarray): Not used.
+            copy (bool): Whether to copy or change in place.
             atomic_number (list): List of array of atomic numbers. Example [np.array([7,1,1,1]), ...].
             sample_weight: Sample weights `(n_samples,)` directly passed to :obj:`Ridge()`. Default is None.
 
         Returns:
-            np.ndarray: Transformed labels of shape `(N, 15)`.
+            np.ndarray: Transformed labels of shape `(n_samples, n_labels)`.
         """
         self.fit(X=X, y=y, atomic_number=atomic_number, sample_weight=sample_weight)
-        return self.transform(X=X, copy=copy, atomic_number=atomic_number)
+        return self.transform(X=X, y=y, copy=copy, atomic_number=atomic_number)
 
-    def transform(self, X, *, copy=None, atomic_number: list = None):
+    def transform(self, y=None, *, X=None, copy=True, atomic_number: list = None):
         r"""Transform all target labels for QM. Requires :obj:`fit()` called previously.
 
         Args:
-            X (np.ndarray): Array of QM unscaled labels of shape `(N, #labels)`.
-            copy (bool): Not fully supported.
+            y (np.ndarray): Array of QM unscaled labels of shape `(n_samples, n_labels)`.
+            X (np.ndarray): Not used.
+            copy (bool): Whether to copy or change in place.
             atomic_number (list): List of atomic numbers for each molecule. E.g. `[np.array([6,1,1,1]), ...]`.
 
         Returns:
-            np.ndarray: Transformed labels of shape `(N, #labels)`.
+            np.ndarray: Transformed labels of shape `(n_samples, n_labels)`.
         """
-        self._check_input(atomic_number, X)
+        labels = self._check_input(atomic_number,  X, y)
 
-        out_labels = []
-        for i, x in enumerate(self.scaler_list):
-            labels = X[:, i:i+1]
-            out_labels.append(x.transform(X=labels, atomic_number=atomic_number))
-
-        out_labels = np.concatenate(out_labels, axis=-1)
+        if copy:
+            out_labels = []
+            for i, x in enumerate(self.scaler_list):
+                out_labels.append(x.transform(X=labels[:, i:i+1], atomic_number=atomic_number, copy=copy))
+            out_labels = np.concatenate(out_labels, axis=-1)
+        else:
+            for i, x in enumerate(self.scaler_list):
+                x.transform(X=labels[:, i:i+1], atomic_number=atomic_number, copy=copy)
+            out_labels = labels
         return out_labels
 
-    def fit(self, X, *, y=None, sample_weight=None, atomic_number=None):
+    def fit(self, y=None, *, X=None, sample_weight=None, atomic_number=None):
         r"""Fit scaling of QM graph labels or targets.
 
         Args:
-            X (np.ndarray): Array of atomic properties of shape `(n_samples, n_properties)`.
-            y (np.ndarray): Ignored.
+            y (np.ndarray): Array of atomic labels of shape `(n_samples, n_labels)`.
+            X (np.ndarray): Ignored.
             atomic_number (list): List of arrays of atomic numbers. Example [np.array([7,1,1,1]), ...].
             sample_weight: Sample weights `(n_samples,)` directly passed to :obj:`Ridge()`. Default is None.
 
         Returns:
             self
         """
-        self._check_input(atomic_number, X)
+        labels = self._check_input(atomic_number, X, y)
 
         for i, x in enumerate(self.scaler_list):
-            labels = X[:, i:i + 1]
-            x.fit(X=labels, y=y, atomic_number=atomic_number, sample_weight=sample_weight)
+            x.fit(labels[:, i:i + 1], atomic_number=atomic_number, sample_weight=sample_weight)
 
         return self
 
-    def inverse_transform(self, X, *, copy: bool = None, atomic_number: list = None):
+    def inverse_transform(self, y=None, *, X=None, copy: bool = True, atomic_number: list = None):
         r"""Back-transform all target labels for QM.
 
         Args:
-            X (np.ndarray): Array of atomic properties of shape `(n_samples, n_properties)`.
-            copy (bool): Not fully supported.
+            y (np.ndarray): Array of atomic labels of shape `(n_samples, n_labels)`.
+            X (np.ndarray): Not used.
+            copy (bool): Whether to copy or change in place.
             atomic_number (list): List of arrays of atomic numbers. Example [np.array([7,1,1,1]), ...].
 
         Returns:
-            np.ndarray: Back-transformed labels of shape `(N, 15)`.
+            np.ndarray: Back-transformed labels of shape `(n_samples, n_labels)`.
         """
-        self._check_input(atomic_number, X)
+        labels = self._check_input(atomic_number, X, y)
 
-        out_labels = []
-        for i, x in enumerate(self.scaler_list):
-            labels = X[:, i:i + 1]
-            out_labels.append(x.inverse_transform(X=labels, atomic_number=atomic_number))
+        if copy:
+            out_labels = []
+            for i, x in enumerate(self.scaler_list):
+                out_labels.append(x.inverse_transform(labels[:, i:i + 1], atomic_number=atomic_number, copy=copy))
+            out_labels = np.concatenate(out_labels, axis=-1)
+        else:
+            for i, x in enumerate(self.scaler_list):
+                x.inverse_transform(labels[:, i:i + 1], atomic_number=atomic_number, copy=copy)
+            out_labels = labels
 
-        out_labels = np.concatenate(out_labels, axis=-1)
         return out_labels
 
-    def _check_input(self, node_number, graph_labels):
+    def _check_input(self, node_number, X, y):
+        assert X is not None or y is not None, "`QMGraphLabelScaler` did not get properties or labels."
+        graph_labels = X if (y is None and X is not None) else y
         assert len(node_number) == len(graph_labels), "`QMGraphLabelScaler` input length does not match."
         assert graph_labels.shape[-1] == len(self.scaler_list), "`QMGraphLabelScaler` got wrong number of labels."
+        return graph_labels
 
     @property
     def scale_(self):
@@ -452,6 +498,7 @@ class QMGraphLabelScaler:
         scaler_conf = config["scaler"]
         for i, x in enumerate(scaler_conf):
             self.scaler_list[i].set_config(x["config"])
+        return self
 
     def save(self, file_path: str):
         """Save scaler serialization to file.
@@ -475,3 +522,4 @@ class QMGraphLabelScaler:
         # Could verify class_name and module_name here.
         self.set_config(full_info["config"])
         self.set_weights(full_info["weights"])
+        return self
\ No newline at end of file
